#!/usr/bin/env python
# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <gogogo.vm@gmail.com>
"""

from __future__ import print_function

import argparse
import sys

import pingparsing

import examplecommon


def parse_option():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-d", "--destination", required=True, help="destination host")
    parser.add_argument(
        "-I", dest="interface", help="interface")

    return parser.parse_args()


def main():
    options = parse_option()

    ping_parser = pingparsing.PingParsing()
    transmitter = pingparsing.PingTransmitter()
    transmitter.destination_host = options.destination
    transmitter.interface = options.interface
    transmitter.waittime = 10
    result = transmitter.ping()
    ping_parser.parse(result.stdout)

    print("# returncode ---")
    print(result.returncode)
    print()
    examplecommon.print_ping_parser(ping_parser)

    return 0


if __name__ == "__main__":
    sys.exit(main())



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <gogogo.vm@gmail.com>
"""

from __future__ import absolute_import

from .error import PingStaticticsHeaderNotFoundError
from .error import EmptyPingStaticticsError

from ._pingparsing import PingParsing
from ._pingtransmitter import (
    PingTransmitter,
    PingResult
)



#!/usr/bin/env python
# encoding: utf-8

from __future__ import unicode_literals
import os
import sys
import subprocess


VERSION = "0.2.0"
README_DIR = ".."
README_WORK_DIR = "."
README_HEADER_PATH = os.path.join(README_WORK_DIR, "README_HEADER.rst")


with open(os.path.join(README_WORK_DIR, "summary.txt")) as f:
    summary = [str(line).strip() for line in f.readlines() if line.strip()]


def main():
    convert_command = "pandoc -f markdown -t rst README.md"
    process = subprocess.Popen(
        convert_command, shell=True,
        stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    ret_stdout, _ret_stderr = process.communicate()

    with open(os.path.join(README_DIR, "README.rst"), "w") as f:
        f.write("\n".join([
            line.rstrip() for line in open(README_HEADER_PATH).readlines()
        ]))
        f.write("\n" * 2)
        f.write("Summary\n")
        f.write("=======\n")
        f.write("\n".join([line.rstrip() for line in summary]))
        f.write("\n" * 2)
        f.write("\n".join([str(line).rstrip()
                           for line in ret_stdout.splitlines()]))
        f.write("\n")

if __name__ == '__main__':
    sys.exit(main())



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <gogogo.vm@gmail.com>
"""

from pingparsing import (
    PingParsing,
    PingTransmitter,
)
import pytest
from typepy.type import RealNumber


@pytest.fixture
def ping_parser():
    return PingParsing()


@pytest.fixture
def transmitter():
    return PingTransmitter()


class Test_PingParse_parse:

    @pytest.mark.xfail
    @pytest.mark.parametrize(["host", "value", "expected"], [
        ["localhost", None, 1],
        ["localhost", 3, 3],
        ["127.0.0.1", 1, 1],
        ["::1", 1, 1],
    ])
    def test_normal(
            self, transmitter, ping_parser, host, value, expected):
        transmitter.destination_host = host
        transmitter.count = value
        transmitter.waittime = value
        result = transmitter.ping()

        ping_parser.parse(result.stdout)

        assert ping_parser.packet_transmit >= expected
        assert RealNumber(ping_parser.packet_receive).is_type()
        assert RealNumber(ping_parser.packet_loss).is_type()
        assert RealNumber(ping_parser.rtt_min).is_type()
        assert RealNumber(ping_parser.rtt_avg).is_type()
        assert RealNumber(ping_parser.rtt_max).is_type()
        assert RealNumber(ping_parser.rtt_mdev).is_type()

    @pytest.mark.xfail
    @pytest.mark.parametrize(["host", "waittime", "expected"], [
        ["localhost", 1, 1],
        ["localhost", 3, 3],
    ])
    def test_normal_waittime(
            self, transmitter, ping_parser, host, waittime, expected):
        transmitter.destination_host = host
        transmitter.waittime = waittime
        result = transmitter.ping()

        ping_parser.parse(result.stdout)

        assert ping_parser.packet_transmit >= expected
        assert RealNumber(ping_parser.packet_receive).is_type()
        assert RealNumber(ping_parser.packet_loss).is_type()
        assert RealNumber(ping_parser.rtt_min).is_type()
        assert RealNumber(ping_parser.rtt_avg).is_type()
        assert RealNumber(ping_parser.rtt_max).is_type()
        assert RealNumber(ping_parser.rtt_mdev).is_type()

    @pytest.mark.xfail
    @pytest.mark.parametrize(["host", "count", "expected"], [
        ["localhost", 1, 1],
        ["localhost", 3, 3],
    ])
    def test_normal_count(
            self, transmitter, ping_parser, host, count, expected):
        transmitter.destination_host = host
        transmitter.count = count
        result = transmitter.ping()

        ping_parser.parse(result.stdout)

        assert ping_parser.packet_transmit == expected
        assert RealNumber(ping_parser.packet_receive).is_type()
        assert RealNumber(ping_parser.packet_loss).is_type()
        assert RealNumber(ping_parser.rtt_min).is_type()
        assert RealNumber(ping_parser.rtt_avg).is_type()
        assert RealNumber(ping_parser.rtt_max).is_type()
        assert RealNumber(ping_parser.rtt_mdev).is_type()



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <gogogo.vm@gmail.com>
"""

from __future__ import absolute_import

import re

import typepy
import pyparsing as pp

from .error import EmptyPingStaticticsError
from .error import PingStaticticsHeaderNotFoundError


def _to_unicode(text):
    try:
        return text.decode("ascii")
    except AttributeError:
        return text


class PingParsing(object):
    """
    Parser class to parsing ping command output.
    """

    def __init__(self):
        self.ping_option = ""

        self.__initialize_parse_result()

    @property
    def packet_transmit(self):
        """
        :return: Number of packets transmitted.
        :rtype: int
        """

        return self.__packet_transmit

    @property
    def packet_receive(self):
        """
        :return: Number of packets received.
        :rtype: int
        """

        return self.__packet_receive

    @property
    def packet_loss(self):
        """
        :return: Percentage of packet loss [%].
        :rtype: float
        """

        return self.__packet_loss

    @property
    def rtt_min(self):
        """
        :return: Minimum round trip time of transmitted ICMP packets [ms].
        :rtype: float
        """

        return self.__rtt_min

    @property
    def rtt_avg(self):
        """
        :return: Average round trip time of transmitted ICMP packets [ms].
        :rtype: float
        """

        return self.__rtt_avg

    @property
    def rtt_max(self):
        """
        :return: Maximum round trip time of transmitted ICMP packets [ms].
        :rtype: float
        """

        return self.__rtt_max

    @property
    def rtt_mdev(self):
        """
        :return: Standard deviation of transmitted ICMP packets (Linux only).
        :rtype: float
        """

        return self.__rtt_mdev

    def as_dict(self):
        """
        :return: Parsed result as a dictionary.
        :rtype: dict
        """

        return {
            "packet_transmit": self.packet_transmit,
            "packet_receive": self.packet_receive,
            "packet_loss": self.packet_loss,
            "rtt_min": self.rtt_min,
            "rtt_avg": self.rtt_avg,
            "rtt_max": self.rtt_max,
            "rtt_mdev": self.rtt_mdev,
        }

    def parse(self, ping_message):
        """
        Parse ping command output.
        You can get parsing results by attributes:

            - :py:attr:`.packet_transmit`
            - :py:attr:`.packet_receive`
            - :py:attr:`.packet_loss`
            - :py:attr:`.rtt_min`
            - :py:attr:`.rtt_avg`
            - :py:attr:`.rtt_max`
            - :py:attr:`.rtt_mdev`

        Or you can get as a dictionary by :py:meth:`.as_dict`

        :param str ping_message: String of ping command output.
        """

        self.__initialize_parse_result()

        if typepy.is_null_string(ping_message):
            return

        try:
            self.__parse_linux_ping(ping_message)
            return
        except PingStaticticsHeaderNotFoundError:
            pass

        self.__parse_windows_ping(ping_message)

    def __find_ststs_head_line_idx(self, line_list, re_stats_header):
        for i, line in enumerate(line_list):
            if re_stats_header.search(line):
                break
        else:
            raise PingStaticticsHeaderNotFoundError(
                "ping statistics not found")

        return i

    def __validate_stats_body(self, body_line_list):
        if typepy.is_empty_sequence(body_line_list):
            raise EmptyPingStaticticsError("ping statistics is empty")

    def __parse_windows_ping(self, ping_message):
        line_list = _to_unicode(ping_message).splitlines()

        i = self.__find_ststs_head_line_idx(
            line_list, re.compile("^Ping statistics for "))

        body_line_list = line_list[i + 1:]
        self.__validate_stats_body(body_line_list)
        packet_line = body_line_list[0].strip()
        packet_pattern = (
            pp.Literal("Packets: Sent = ") +
            pp.Word(pp.nums) +
            pp.Literal(", Received = ") +
            pp.Word(pp.nums) +
            pp.Literal(", Lost = ") +
            pp.Word(pp.nums) + "(" +
            pp.Word(pp.nums + ".")
        )
        parse_list = packet_pattern.parseString(_to_unicode(packet_line))
        self.__packet_transmit = int(parse_list[1])
        self.__packet_receive = int(parse_list[3])
        self.__packet_loss = float(parse_list[7])

        try:
            rtt_line = body_line_list[2].strip()
        except IndexError:
            return
        if typepy.is_null_string(rtt_line):
            return
        rtt_pattern = (
            pp.Literal("Minimum = ") +
            pp.Word(pp.nums) +
            pp.Literal("ms, Maximum = ") +
            pp.Word(pp.nums) +
            pp.Literal("ms, Average = ") +
            pp.Word(pp.nums)
        )
        try:
            parse_list = rtt_pattern.parseString(_to_unicode(rtt_line))
        except pp.ParseBaseException:
            return

        self.__rtt_min = float(parse_list[1])
        self.__rtt_avg = float(parse_list[5])
        self.__rtt_max = float(parse_list[3])

    def __parse_linux_ping(self, ping_message):
        line_list = _to_unicode(ping_message).splitlines()

        i = self.__find_ststs_head_line_idx(
            line_list, re.compile("--- .* ping statistics ---"))

        body_line_list = line_list[i + 1:]
        self.__validate_stats_body(body_line_list)

        packet_line = body_line_list[0]
        packet_pattern = (
            pp.Word(pp.nums) +
            pp.Literal("packets transmitted,") +
            pp.Word(pp.nums) +
            pp.Literal("received,") +
            pp.SkipTo(pp.Word(pp.nums + ".%") + pp.Literal("packet loss")) +
            pp.Word(pp.nums + ".") +
            pp.Literal("% packet loss")
        )
        parse_list = packet_pattern.parseString(_to_unicode(packet_line))
        self.__packet_transmit = int(parse_list[0])
        self.__packet_receive = int(parse_list[2])
        self.__packet_loss = float(parse_list[-2])

        try:
            rtt_line = body_line_list[1]
        except IndexError:
            return
        if typepy.is_null_string(rtt_line):
            return

        rtt_pattern = (
            pp.Literal("rtt min/avg/max/mdev =") +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") +
            pp.Word(pp.nums + "ms")
        )
        try:
            parse_list = rtt_pattern.parseString(_to_unicode(rtt_line))
        except pp.ParseBaseException:
            return

        self.__rtt_min = float(parse_list[1])
        self.__rtt_avg = float(parse_list[3])
        self.__rtt_max = float(parse_list[5])
        self.__rtt_mdev = float(parse_list[7])

    def __initialize_parse_result(self):
        self.__packet_transmit = None
        self.__packet_receive = None
        self.__packet_loss = None
        self.__rtt_min = None
        self.__rtt_avg = None
        self.__rtt_max = None
        self.__rtt_mdev = None



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <gogogo.vm@gmail.com>
"""

import itertools

from pingparsing import (
    EmptyPingStaticticsError,
    PingParsing,
    PingResult,
)
import pytest
import six


@pytest.fixture
def ping_text():
    return six.b("""
PING google.com (216.58.196.238) 56(84) bytes of data.

--- google.com ping statistics ---
60 packets transmitted, 60 received, 0% packet loss, time 59153ms
rtt min/avg/max/mdev = 61.425/99.731/212.597/27.566 ms
""")

# ping google.com -q -c 60:
#   - Debian 8.2 w/ iputils-ping 20121221-5+b2
#   - Debian 5.0.10 w/ iputils-ping 20071127-1+lenny1
PING_DEBIAN_SUCCESS = six.b("""PING google.com (216.58.196.238) 56(84) bytes of data.

--- google.com ping statistics ---
60 packets transmitted, 60 received, 0% packet loss, time 59153ms
rtt min/avg/max/mdev = 61.425/99.731/212.597/27.566 ms
""")


PING_DEBIAN_UNREACHABLE_0 = """
PING 192.168.207.100 (192.168.207.100) 56(84) bytes of data.

--- 192.168.207.100 ping statistics ---
5 packets transmitted, 0 received, 100% packet loss, time 4009ms
"""
PING_DEBIAN_UNREACHABLE_1 = PING_DEBIAN_UNREACHABLE_0 + "\n"
PING_DEBIAN_UNREACHABLE_2 = PING_DEBIAN_UNREACHABLE_1 + "\n"
PING_DEBIAN_UNREACHABLE_3 = PING_DEBIAN_UNREACHABLE_0 + "\npipe 4\n"

PING_FEDORA_LOSS = six.b("""
PING 192.168.0.1 (192.168.0.1) 56(84) bytes of data.

--- 192.168.0.1 ping statistics ---
1688 packets transmitted, 1553 received, +1 duplicates, 7% packet loss, time 2987ms
rtt min/avg/max/mdev = 0.282/0.642/11.699/0.699 ms, pipe 2, ipg/ewma 1.770/0.782 ms
""")

PING_FEDORA_UNREACHABLE = """
PING 192.168.207.100 (192.168.207.100) 56(84) bytes of data.
From 192.168.207.128 icmp_seq=1 Destination Host Unreachable
From 192.168.207.128 icmp_seq=2 Destination Host Unreachable
From 192.168.207.128 icmp_seq=3 Destination Host Unreachable
From 192.168.207.128 icmp_seq=4 Destination Host Unreachable
From 192.168.207.128 icmp_seq=5 Destination Host Unreachable

--- 192.168.207.100 ping statistics ---
5 packets transmitted, 0 received, +5 errors, 100% packet loss, time 4003ms
"""

PING_FEDORA_EMPTY_BODY = six.b("""
PING 192.168.0.1 (192.168.0.1) 56(84) bytes of data.

--- 192.168.0.1 ping statistics ---
""")


# ping google.com -n 10:
#   Windows 7 SP1
PING_WINDOWS_SUCCESS = six.b("""
Pinging google.com [216.58.196.238] with 32 bytes of data:
Reply from 216.58.196.238: bytes=32 time=87ms TTL=51
Reply from 216.58.196.238: bytes=32 time=97ms TTL=51
Reply from 216.58.196.238: bytes=32 time=56ms TTL=51
Reply from 216.58.196.238: bytes=32 time=95ms TTL=51
Reply from 216.58.196.238: bytes=32 time=194ms TTL=51
Reply from 216.58.196.238: bytes=32 time=98ms TTL=51
Reply from 216.58.196.238: bytes=32 time=93ms TTL=51
Reply from 216.58.196.238: bytes=32 time=96ms TTL=51
Reply from 216.58.196.238: bytes=32 time=96ms TTL=51
Reply from 216.58.196.238: bytes=32 time=165ms TTL=51

Ping statistics for 216.58.196.238:
    Packets: Sent = 10, Received = 10, Lost = 0 (0% loss),
Approximate round trip times in milli-seconds:
    Minimum = 56ms, Maximum = 194ms, Average = 107ms
""")


PING_WINDOWS_UNREACHABLE_0 = """

Pinging 192.168.207.100 with 32 bytes of data:
Request timed out.
Request timed out.
Request timed out.
Request timed out.

Ping statistics for 192.168.207.100:
    Packets: Sent = 4, Received = 0, Lost = 4 (100% loss),
"""
PING_WINDOWS_UNREACHABLE_1 = PING_WINDOWS_UNREACHABLE_0 + "\n"
PING_WINDOWS_UNREACHABLE_2 = PING_WINDOWS_UNREACHABLE_1 + "\n"


PING_WINDOWS_INVALID = """

Pinging 192.168.207.100 with 32 bytes of data:
Request timed out.
Request timed out.
Request timed out.
Request timed out.

Ping statistics for 192.168.207.100:
"""


@pytest.fixture
def ping_parser():
    return PingParsing()


class Test_PingParsing_parse:

    @pytest.mark.parametrize(["ping_text", "expected"], [
        [
            PING_DEBIAN_SUCCESS,
            {
                "packet_transmit": 60,
                "packet_receive": 60,
                "packet_loss": 0.0,
                "rtt_min": 61.425,
                "rtt_avg": 99.731,
                "rtt_max": 212.597,
                "rtt_mdev": 27.566,
            }
        ],
        [
            PING_FEDORA_LOSS,
            {
                'packet_loss': 7.0,
                'packet_receive': 1553,
                'packet_transmit': 1688,
                'rtt_min': 0.282,
                'rtt_max': 11.699,
                'rtt_mdev': 0.699,
                'rtt_avg': 0.642,
            }
        ],
        [
            PING_FEDORA_UNREACHABLE,
            {
                "packet_transmit": 5,
                "packet_receive": 0,
                "packet_loss": 100.0,
                "rtt_min": None,
                "rtt_avg": None,
                "rtt_max": None,
                "rtt_mdev": None,
            }
        ],
        [
            PING_WINDOWS_SUCCESS,
            {
                "packet_transmit": 10,
                "packet_receive": 10,
                "packet_loss": 0.0,
                "rtt_min": 56,
                "rtt_avg": 107,
                "rtt_max": 194,
                "rtt_mdev": None,
            }
        ],
    ] + list(itertools.product(
        [
            PING_DEBIAN_UNREACHABLE_0,
            PING_DEBIAN_UNREACHABLE_1,
            PING_DEBIAN_UNREACHABLE_2,
        ],
        [{
            "packet_transmit": 5,
            "packet_receive": 0,
            "packet_loss": 100.0,
            "rtt_min": None,
            "rtt_avg": None,
            "rtt_max": None,
            "rtt_mdev": None,
        }]
    )) + list(itertools.product(
        [
            PING_WINDOWS_UNREACHABLE_0,
            PING_WINDOWS_UNREACHABLE_1,
            PING_WINDOWS_UNREACHABLE_2,
        ],
        [{
            "packet_transmit": 4,
            "packet_receive": 0,
            "packet_loss": 100.0,
            "rtt_min": None,
            "rtt_avg": None,
            "rtt_max": None,
            "rtt_mdev": None,
        }]
    ))
    )
    def test_normal_text(self, ping_parser, ping_text, expected):
        ping_parser.parse(ping_text)

        assert ping_parser.as_dict() == expected

    @pytest.mark.parametrize(["pingresult", "expected"], [
        [
            PingResult(PING_DEBIAN_SUCCESS, "", 0),
            {
                "packet_transmit": 60,
                "packet_receive": 60,
                "packet_loss": 0.0,
                "rtt_min": 61.425,
                "rtt_avg": 99.731,
                "rtt_max": 212.597,
                "rtt_mdev": 27.566,
            }
        ]
    ])
    def test_normal_pingresult(self, ping_parser, pingresult, expected):
        ping_parser.parse(pingresult.stdout)

        assert ping_parser.as_dict() == expected

    def test_empty(self, ping_parser, ping_text):
        ping_parser.parse(ping_text)
        ping_parser.parse("")

        assert ping_parser.packet_transmit is None
        assert ping_parser.packet_receive is None
        assert ping_parser.packet_loss is None
        assert ping_parser.rtt_min is None
        assert ping_parser.rtt_avg is None
        assert ping_parser.rtt_max is None
        assert ping_parser.rtt_mdev is None

    @pytest.mark.parametrize(["ping_text", "expected"], [
        [PING_FEDORA_EMPTY_BODY, EmptyPingStaticticsError],
        [PING_WINDOWS_INVALID, EmptyPingStaticticsError],
    ])
    def test_exception(self, ping_parser, ping_text, expected):
        with pytest.raises(expected):
            ping_parser.parse(ping_text)



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <gogogo.vm@gmail.com>
"""

from __future__ import absolute_import


class PingStaticticsHeaderNotFoundError(Exception):
    """
    Exception raised when a ping statistics header is not found in
    a parsing text.
    """


class EmptyPingStaticticsError(Exception):
    """
    Exception raised when a ping statistics is empty in a parsing text.
    """



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <gogogo.vm@gmail.com>
"""

import io
import os.path
import sys

import setuptools


REQUIREMENT_DIR = "requirements"

needs_pytest = set(["pytest", "test", "ptr"]).intersection(sys.argv)
pytest_runner = ["pytest-runner"] if needs_pytest else []


with io.open("README.rst", encoding="utf8") as fp:
    long_description = fp.read()

with io.open(os.path.join(
        "docs", "pages", "introduction", "summary.txt"), encoding="utf8") as f:
    summary = f.read()

with open(os.path.join(REQUIREMENT_DIR, "requirements.txt")) as f:
    install_requires = [line.strip() for line in f if line.strip()]

if sys.version_info.major < 3 or sys.version_info.minor < 4:
    install_requires.append("ipaddress")

with open(os.path.join(REQUIREMENT_DIR, "test_requirements.txt")) as f:
    tests_require = [line.strip() for line in f if line.strip()]

setuptools.setup(
    name="pingparsing",
    version="0.5.0",
    author="Tsuyoshi Hombashi",
    author_email="gogogo.vm@gmail.com",
    url="https://github.com/thombashi/pingparsing",
    keywords=["ping", "parser", "transmitter"],
    license="MIT License",
    description=summary,
    long_description=long_description,
    include_package_data=True,
    install_requires=install_requires,
    packages=setuptools.find_packages(exclude=["test*"]),
    setup_requires=pytest_runner,
    tests_require=tests_require,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Networking",
    ],
)
